"""

How to disable authentication of internal principals
****************************************************

:Test-Layer: functional

If we want to disable 'default' principals (users) defined in
``site.zcml`` or similar from logging in, we can use
``megrok.login.strict()``. This marker directive takes care, so that
only explicitly added users are allowed to log in.

The background: ``megrok.login`` by default installs *two* authenticator
plugins for your site:

 1) a usual ``PrincipalFolder`` instance which is empty in the
    beginning.

 2) a fallback folder that authenticates against principals held in
    the central principal registry.

The latter allows login of all principals defined in your site.zcml or
other ZCML configurations read on startup. The fallback-folder is
read-only. The common manager login for the whole instance is an
example for such an 'internal' principal.

When using ``megrok.login.strict()``, the second folder is not created
and the normally working accounts from the global principal registry
will not work.

We create an instance of App and store it in the ZODB::

  >>> from megrok.login.tests.strict import StrictApp
  >>> root = getRootFolder()
  >>> root['app'] = StrictApp()

Now, when we try to access the `index` view of the app, we'll get a
login page::
  
  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.open('http://localhost/app')
  >>> print browser.contents
  <!DOCTYPE html ...
  Please provide Login Information...
  <input type="text" name="login" id="login" />
  ...

When we enter the normal manager credentials, this will not let us
in. Instead we will again see the login page::
  
  >>> browser.getControl('User Name').value = 'mgr'
  >>> browser.getControl('Password').value = 'mgrpw'
  >>> browser.getControl('Log in').click()

  >>> print browser.contents
  <!DOCTYPE html ...
  Please provide Login Information...
  <input type="text" name="login" id="login" />
  ...

Before we can login, we have to add a user to the one and only
principal folder of our app.

First we create a new principal (user)::

  >>> from zope.app.authentication.principalfolder import InternalPrincipal
  >>> bob = InternalPrincipal('bob', 'bobpw', 'bob')

We add bob to our principal folder::

  >>> app = root['app']
  >>> sm = app.getSiteManager()
  >>> pau = sm['megrok_login_pau']
  >>> principal_folder = pau['principals']
  >>> principal_folder['bob'] = bob

We grant `bob` the permission to view our app::

  >>> from zope.securitypolicy.interfaces import IPrincipalPermissionManager
  >>> perm_mgr = IPrincipalPermissionManager(app)
  >>> perm_mgr.grantPermissionToPrincipal(
  ...   'app.ManageStrict', principal_folder.prefix + 'bob')

Now we authenticate as Bob::

  >>> browser.getControl('User Name').value = 'bob'
  >>> browser.getControl('Password').value = 'bobpw'
  >>> browser.getControl('Log in').click()

We are redirected to the page we wanted to access in the beginning::

  >>> print browser.contents
  Hi from strict app!

"""
import grok
import megrok.login

class ManageApp(grok.Permission):
    grok.name('app.ManageStrict')

class StrictApp(grok.Application, grok.Container):
    megrok.login.enable()
    megrok.login.strict()

class Index(grok.View):
    grok.require('app.ManageStrict')

    def render(self):
        return "Hi from strict app!"



import os.path
import megrok.login
from zope.app.testing.functional import ZCMLLayer

ftesting_zcml = os.path.abspath(os.path.join(os.path.dirname(
            megrok.login.__file__), 'ftesting.zcml'))
FunctionalLayer = ZCMLLayer(ftesting_zcml, __name__, 'FunctionalLayer',
                            allow_teardown=True)



import martian
from martian.error import GrokImportError

class enable(martian.MarkerDirective):
    scope = martian.CLASS
    store = martian.ONCE

class viewname(martian.Directive):
    scope = martian.CLASS
    store = martian.ONCE
    default = u'loginForm.html'
    
    def factory(self, viewname=None):
        if viewname is None:
            viewname = self.default
        if martian.util.not_unicode_or_ascii(viewname):
            raise GrokImportError(
                "You can only pass unicode, None or ASCII "
                "to the 'megrok.login.viewname' directive.")
        return viewname

class strict(martian.MarkerDirective):
    scope = martian.CLASS
    store = martian.ONCE

class autoregister(martian.MultipleTimesDirective):
    scope = martian.CLASS

class setup(martian.Directive):
    scope = martian.CLASS
    store = martian.ONCE
    default = None

    # XXX: Check, that the given callable exists



"""

:Test-Layer: functional

If the `megrok.login.enable` directive is not used, we get the
standard behaviour, a HTTP error: 401 Unauthorized when trying to
access a protected view.

We create an instance of App and store it in the ZODB::

  >>> from megrok.login.tests.unset import App
  >>> root = getRootFolder()
  >>> root['app'] = App()

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.open('http://localhost/app')
  Traceback (most recent call last):
  ...
  httperror_seek_wrapper: HTTP Error 401: Unauthorized

We can, however, use basic-auth::

  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  >>> browser.open('http://localhost/app')
  >>> browser.headers['status']
  '200 Ok'
  

"""
import grok

class ManageApp(grok.Permission):
    grok.name('app.ManageRaw')

class App(grok.Application, grok.Container):
    pass

class Index(grok.View):
    grok.require('app.ManageRaw')

    def render(self):
        return "Hi!"



##############################################################################
#
# Copyright (c) 2008 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from zope.app.authentication.interfaces import IAuthenticatorPlugin
from zope.app.authentication.interfaces import IQuerySchemaSearch
from zope.app.authentication.principalfolder import (PrincipalInfo,
                                                     PrincipalFolder,
                                                     InternalPrincipal)
from zope.app.authentication.principalfolder import ISearchSchema
from zope.app.container.interfaces import DuplicateIDError
from zope.app.security.principalregistry import principalRegistry
from zope.securitypolicy.interfaces import IPrincipalPermissionManager
from zope.interface import implements

class PrincipalRegistryAuthenticator(object):
    """An authenticator plugin, that authenticates principals against
    the global principal registry.

    This authenticator does not support own prefixes, because the
    prefix of its principals is already defined in another place
    (site.zcml). Therefore we get and give back IDs as they are.
    """

    implements(IAuthenticatorPlugin, IQuerySchemaSearch)

    schema = ISearchSchema

    def authenticateCredentials(self, credentials):
        """Return principal info if credentials can be authenticated
        """
        if not isinstance(credentials, dict):
            return None
        if not ('login' in credentials and 'password' in credentials):
            return None
        principal = None
        login, password = credentials['login'], credentials['password']
        try:
            principal = principalRegistry.getPrincipalByLogin(login)
        except KeyError:
            return
        if principal and principal.validate(password):
            return PrincipalInfo(u''+principal.id,
                                 principal.getLogin(),
                                 principal.title,
                                 principal.description)
        return

    def principalInfo(self, id):
        principal = principalRegistry.getPrincipal(id)
        if principal is not None:
            return PrincipalInfo(u''+principal.id,
                                 principal.getLogin(),
                                 principal.title,
                                 principal.description)


    def search(self, query, start=None, batch_size=None):
        """Search through this principal provider.
        """
        search = query.get('search')
        if search is None:
            return
        search = search.lower()
        n = 1
        values = [x for x in principalRegistry.getPrincipals('')
                  if x is not None]
        values.sort(cmp=lambda x,y: cmp(str(x.id), str(y.id)))
        for i, value in enumerate(values):
            if (search in value.id.lower() or
                search in value.title.lower() or
                search in value.description.lower() or
                search in value.getLogin().lower()):
                if not ((start is not None and i < start)
                        or (batch_size is not None and n > batch_size)):
                    n += 1
                    yield u''+value.__name__


class AutoRegisteringPrincipalFolder(PrincipalFolder):
    """A principalfolder, that autoregisters new principals.
    """
    def __init__(self, prefix='', autopermissions=()):
        super(AutoRegisteringPrincipalFolder, self).__init__(prefix)
        self.__id_by_login = self._newContainerData()
        self.auto_permissions = autopermissions
 
    def authenticateCredentials(self, credentials):
        """An authentication that adds the credentials if possible.
        """
        if not isinstance(credentials, dict):
            return None
        if not ('login' in credentials and 'password' in credentials):
            return None
        id = self.__id_by_login.get(credentials['login'])
        if id is None:
            # Create new principal...
            id = credentials['login']
            principal = InternalPrincipal(id, credentials['password'], id)
            try:
                self[id] = principal
                perm_mgr = IPrincipalPermissionManager(
                    self.__parent__.__parent__.__parent__)
                for perm in self.auto_permissions:
                    perm_mgr.grantPermissionToPrincipal(
                        perm, self.prefix + id)
            except DuplicateIDError:
                pass
        internal = self[id]
        if not internal.checkPassword(credentials["password"]):
            return None
        return PrincipalInfo(self.prefix + id, internal.login, internal.title,
                             internal.description)




"""
How to setup a cutomized PAU with ``megrok.login``
**************************************************

:Test-Layer: functional

You can setup the Pluggable Authentication Utility (PAU) completely
yourself. We defined a function `customPAUSetup` below, which performs
exactly that task.

We use::

  megrok.login.setup(<callable>)

to setup the PAU using this function.

As with all other directives of ``megrok.login``, we also have to set::

  megrok.login.enable()

to generally enable the ``megrok.login`` machinery explicitly.

We will see how this works in action and start by creating an
application instance::

  >>> from megrok.login.tests.custompausetup import CustomSetupApp
  >>> app = CustomSetupApp()

To setup the PAU, the app must become a site (it reacts on
IObjectAdded events). This is performed for instance by adding it to
the ZODB root folder::

  >>> root = getRootFolder()
  >>> root['app'] = app
  Custom PAU setup performed for <...CustomSetupApp object at 0x...>

The installed PAU is indeed the one, we configured below::

  >>> sm = app.getSiteManager()
  >>> sorted(list(sm['megrok_login_pau']))
  [u'custom_session', u'customprincipals', u'my_readonly_principals']

The installed setup is active. We can login via a login screen::

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.open('http://localhost/app')
  >>> print browser.contents
  <!DOCTYPE html ...
  Please provide Login Information...
  <input type="text" name="login" id="login" />
  ...

We enter the credentials::
  
  >>> browser.getControl('User Name').value = 'mgr'
  >>> browser.getControl('Password').value = 'mgrpw'
  >>> browser.getControl('Log in').click()

We are redirected to the page we wanted to access in the beginning::

  >>> print browser.contents
  Hi from custom setup app!

Note, that the installed PAU (as any other PAU installed by
``megrok.login``) does only handle access to `app` or objects 'below'
`app`. It is a *local* utility. We are not authenticated for the root
folder, for example, which is handled by a different PAU::

  >>> browser.open('http://localhost')
  >>> print browser.contents
  <!DOCTYPE html...
  User:...
  Unauthenticated User...
  </html>

"""
import grok
import megrok.login

from zope.app.authentication import PluggableAuthentication
from zope.app.authentication.principalfolder import PrincipalFolder
from zope.app.authentication.session import SessionCredentialsPlugin
from zope.app.security.interfaces import IAuthentication
from megrok.login.authplugins import (PrincipalRegistryAuthenticator,
                                      AutoRegisteringPrincipalFolder)

def customPAUSetup(site, pau, viewname=None, strict=None, autoregister=None):
    """Setup our own PAU.

    This setup performs more or less the same steps as a default
    setup. You are free to change it as you like and build completely
    different setups.
    """
    print "Custom PAU setup performed for %s" % site
    if len(autoregister) > 0 :
        pau['customprincipals'] = AutoRegisteringPrincipalFolder(
            autopermissions = autoregister)
    else:
        pau['customprincipals'] = PrincipalFolder()
    pau.authenticatorPlugins = ('customprincipals', )
    if strict is False:
        pau['my_readonly_principals'] = PrincipalRegistryAuthenticator()
        pau.authenticatorPlugins = ('cutomprincipals', 'my_readonly_principals')

    pau['custom_session'] = session = SessionCredentialsPlugin()
    pau.credentialsPlugins = ('No Challenge if Authenticated',
                              'custom_session',)
    session.loginpagename = viewname
    return

    
class ManageCustomSetupApp(grok.Permission):
    grok.name('app.manageCustomSetupApp')

class CustomSetupApp(grok.Application, grok.Container):
    megrok.login.enable()
    megrok.login.setup(customPAUSetup)

class Index(grok.View):
    grok.require('app.manageCustomSetupApp')

    def render(self):
        return "Hi from custom setup app!"




import re
from zope.testing import module, renormalizing
from zope.app.testing.functional import FunctionalTestSetup
from z3c.testsetup import register_all_tests
from megrok.login.tests import FunctionalLayer

checker = renormalizing.RENormalizing([
    # Relevant normalizers from zope.testing.testrunner.tests:
    (re.compile(r'\d+[.]\d\d\d seconds'), 'N.NNN seconds'),
    # Our own one to work around
    # http://reinout.vanrees.org/weblog/2009/07/16/invisible-test-diff.html:
    (re.compile(r'.*1034h'), ''),
    (re.compile(r'httperror_seek_wrapper:'), 'HTTPError:' )
    ])

def setUp(test):
    if test.filename.endswith('.txt'):
        module.setUp(test, '__main__')
    FunctionalTestSetup().setUp()
    
def tearDown(test):
    FunctionalTestSetup().tearDown()
    if test.filename.endswith('.txt'):
        module.tearDown(test)

test_suite = register_all_tests('megrok.login', layer=FunctionalLayer,
                                fextensions=['.txt', '.py'],
                                fsetup=setUp,
                                fteardown=tearDown,
                                checker=checker,
                                )



"""
How to setup a customized login page
************************************

:Test-Layer: functional

Using the `megrok.login.viewname` directive, you can specify your own
login page. If you do not use it, a default page is used.

If the `megrok.login.enable` directive is used, we get a login page
when trying to access a protected view.

If - additionally - a `megrok.login.viewname` directive is declared,
then this view will be used as the login page.

We create an instance of ``CustomLoginApp`` and store it in the ZODB::

  >>> from megrok.login.tests.customlogin import CustomLoginApp
  >>> root = getRootFolder()
  >>> root['app'] = CustomLoginApp()

Now, when we try to access the `index` view of the app, we'll get a
login page::
  
  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.open('http://localhost/app')
  >>> print browser.contents
  <html>...
  <h1>Custom Login Page</h1>...
  <input type="text" name="login" id="login" />
  ...

We enter the credentials::
  
  >>> browser.getControl(name='login').value = 'mgr'
  >>> browser.getControl(name='password').value = 'mgrpw'
  >>> browser.getControl('Log in').click()

We are redirected to the page we wanted to access in the beginning::

  >>> print browser.contents
  Hi after custom login!

  
Login view fields
-----------------
  
For the authentication it is essential, that your login form provides:

 * a field named `login` with the username (principal title)

 * a field named `password` with the password.

The pluggable authentication utility will lookup those fields.

If the password is wrong (or the username does not exist), we will again
be redirected to the login page.


Context of login views
----------------------

As with all views, also your login view is bound to the types of
objects declared with `grok.context`. You can setup different login
views for different types of objects, but if you want all kinds of
objects to be handled by your form, then you should bind to context
`Interface` as shown below.


"""
import grok
import megrok.login
from zope.interface import Interface

class ManageApp(grok.Permission):
    grok.name('app.ManageCustomLogin')

class CustomLoginApp(grok.Application, grok.Container):
    """An application that provides a custom login page.
    """
    megrok.login.enable()
    megrok.login.viewname('login') # Look for a view named 'login'
                                   # when we need authentication.

class Index(grok.View):
    grok.require('app.ManageCustomLogin')

    def render(self):
        return "Hi after custom login!"

class Login(grok.View):
    """A login view.

    See also `login.pt` in `customlogin_templates`
    """
    grok.context(Interface) # We bind to all possible contexts

    def update(self, camefrom=None, SUBMIT=None):
        self.camefrom=camefrom
        if SUBMIT is not None and camefrom is not None:
            # The credentials were entered. Go back. If the entered
            # credentials are not valid, another redirect will happen
            # to this view.
            self.redirect(camefrom)
        return



from setuptools import setup, find_packages
import os

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

tests_require = [
    'zope.testing',
    'zope.app.testing',
    'zope.testbrowser',
    ]

long_description = (
    read('README.txt')
    + '\n\n'
    + 'Detailed Documentation\n'
    + '**********************\n'
    + '\n'
    + read('src', 'megrok', 'login', 'README.txt')
    + '\n\n'
    + read('CHANGES.txt')
    + '\n\n'
    + 'Download\n'
    + '********\n'
    )

setup(
    name='megrok.login',
    version='0.2',
    author='Uli Fouquet and the Zope Community',
    author_email='uli@gnufix.de',
    url = 'http://pypi.python.org/pypi/megrok.login',
    description='Providing login screens for your Grok apps made easy.',
    long_description=long_description,
    license='ZPL 2.1',
    keywords="zope3 zope login grok security PAU",
    classifiers=['Development Status :: 3 - Alpha',
                 'Environment :: Web Environment',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Zope Public License',
                 'Programming Language :: Python',
                 'Operating System :: OS Independent',
                 'Framework :: Zope3',
                 ],

    packages=find_packages('src'),
    package_dir = {'': 'src'},
    namespace_packages = ['megrok'],
    include_package_data = True,
    zip_safe=False,
    install_requires=['setuptools',
                      'grok',
                      'martian',
                      'zope.app.authentication',
                      'zope.app.security',
                      'zope.app.container',
                      'zope.component',
                      'zope.interface',
                      'zope.securitypolicy',
                      ],
    tests_require = tests_require,
    extras_require = dict(test=tests_require),
)



"""
How to enable session based authentication
******************************************

:Test-Layer: functional

If the `megrok.login.enable` directive is used, we get a login page
when trying to access a protected view.

We create an instance of App and store it in the ZODB::

  >>> from megrok.login.tests.simple import SimpleApp
  >>> root = getRootFolder()
  >>> root['simpleapp'] = SimpleApp()

Now, when we try to access the `index` view of the app, we'll get a
login page::
  
  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.open('http://localhost/simpleapp')
  >>> print browser.contents
  <!DOCTYPE html ...
  Please provide Login Information...
  <input type="text" name="login" id="login" />
  ...

We enter the credentials::
  
  >>> browser.getControl('User Name').value = 'mgr'
  >>> browser.getControl('Password').value = 'mgrpw'
  >>> browser.getControl('Log in').click()

We are redirected to the page we wanted to access in the beginning::

  >>> print browser.contents
  Hi!

"""
import grok
import megrok.login

class ManageApp(grok.Permission):
    grok.name('app.ManageSimple')

class SimpleApp(grok.Application, grok.Container):
    megrok.login.enable()

class Index(grok.View):
    grok.require('app.ManageSimple')

    def render(self):
        return "Hi!"



import martian
import grok
import megrok.login
from zope import component

from zope.app.authentication import PluggableAuthentication
from zope.app.authentication.principalfolder import PrincipalFolder
from zope.app.authentication.session import SessionCredentialsPlugin
from zope.app.security.interfaces import IAuthentication
from megrok.login.authplugins import (PrincipalRegistryAuthenticator,
                                      AutoRegisteringPrincipalFolder)

class ApplicationGrokker(martian.ClassGrokker):
    martian.component(grok.Site)
    martian.priority(100)
    martian.directive(megrok.login.enable, default=False)
    martian.directive(megrok.login.viewname, default=u'loginForm.html')
    martian.directive(megrok.login.strict, default=False)
    martian.directive(megrok.login.autoregister, default=u'')
    martian.directive(megrok.login.setup, default=None)
    
    def execute(self, factory, config, enable, viewname, strict,
                autoregister, setup, **kw):
        if enable is False:
            return False
        # XXX: check autoregister values
        adapts = (factory, grok.IObjectAddedEvent)
        config.action(
            discriminator=None,
            callable=component.provideHandler,
            args=(authenticationSubscriber, adapts)
            )
        return True

def authenticationSubscriber(site, event):
    try:
        # Conditional import. Newer versions of grok do not provide
        # `setupUtility` any more...
        from grok.meta import setupUtility
    except ImportError:
        from grokcore.site.interfaces import IUtilityInstaller
        setupUtility = component.getUtility(IUtilityInstaller)
    setupUtility(site, PluggableAuthentication(), IAuthentication,
                 setup=setupPAU,
                 name_in_container='megrok_login_pau')
        


def setupPAU(pau):
    """Callback to setup the Pluggable Authentication Utility """
    site = pau.__parent__.__parent__
    setup = megrok.login.component.setup.bind().get(site)
    viewname = megrok.login.component.viewname.bind().get(site)
    strict = megrok.login.component.strict.bind().get(site)
    autoregister = megrok.login.component.autoregister.bind().get(site)

    if setup is not None:
        # Call a customer defined callable.
        result = setup(pau, viewname=viewname, strict=strict,
                       autoregister=autoregister)
        return result
    
    if len(autoregister) > 0 :
        pau['principals'] = AutoRegisteringPrincipalFolder(
            autopermissions = autoregister)
    else:
        pau['principals'] = PrincipalFolder()
    pau.authenticatorPlugins = ('principals', )
    if strict is False:
        pau['readonly_principals'] = PrincipalRegistryAuthenticator()
        pau.authenticatorPlugins = ('principals', 'readonly_principals')

    pau['session'] = session = SessionCredentialsPlugin()
    pau.credentialsPlugins = ('No Challenge if Authenticated', 'session',)
    session.loginpagename = viewname
    return None



"""
How to setup auto registering authentication
********************************************

:Test-Layer: functional

When (beside `megrok.login.enable()`, which is always necessary), the
`megrok.login.autoregister()` directive is used, then all credentials
that contain a new username are automatically registered and accepted
when logging in.

The `autoregister` directive expects a permission name as
argument. This permission is granted to each auto-registered user. You
can also use this directive multiple times to grant more than one
permission.

We create an instance of ``AutoRegisterApp`` and store it in the ZODB::

  >>> from megrok.login.tests.autoregister import AutoRegisterApp
  >>> root = getRootFolder()
  >>> root['app'] = AutoRegisterApp()

Now, when we try to access the `index` view of the app, we'll get a
login page::
  
  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.open('http://localhost/app')
  >>> print browser.contents
  <!DOCTYPE html ...
  Please provide Login Information...
  <input type="text" name="login" id="login" />
  ...

We enter the credentials::
  
  >>> browser.getControl('User Name').value = 'bob'
  >>> browser.getControl('Password').value = 'bobspw'
  >>> browser.getControl('Log in').click()

We are redirected to the page we wanted to access in the beginning::

  >>> print browser.contents
  Hi autoregistered user!

We can also logout. This is managed by a specialized view (see
below)::

  >>> browser.open('http://localhost/app/@@logout')
  >>> print browser.contents
  You are logged out.

This is not a lie. When we try to watch our app, we will be asked for
a password again::

  >>> browser.open('http://localhost/app')
  >>> print browser.contents
  <!DOCTYPE html ...
  Please provide Login Information...
  <input type="text" name="login" id="login" />
  ...

If we try to login now with the wrong credentials, we will not get
through::

  >>> browser.getControl('User Name').value = 'bob'
  >>> browser.getControl('Password').value = 'notbobspw'
  >>> browser.getControl('Log in').click()
  >>> print browser.contents
  <!DOCTYPE html ...
  Please provide Login Information...
  <input type="text" name="login" id="login" />
  ...

The correct credentials, of course, will work::

  >>> browser.getControl('User Name').value = 'bob'
  >>> browser.getControl('Password').value = 'bobspw'
  >>> browser.getControl('Log in').click()
  >>> print browser.contents
  Hi autoregistered user!

  
"""
import grok
import megrok.login
from zope.app.security.interfaces import IAuthentication
from zope.component import getUtility

class ManageApp(grok.Permission):
    grok.name('app.ManageAutoRegister')

class AutoRegisterApp(grok.Application, grok.Container):
    megrok.login.enable()
    # We grant this permission to autoregistered users.
    megrok.login.autoregister('app.ManageAutoRegister')

class Index(grok.View):
    grok.require('app.ManageAutoRegister')

    def render(self):
        return "Hi autoregistered user!"

class Logout(grok.View):
    """A logout screen.
    """
    
    def render(self):
        # The session plugin which is automatically created, is always
        # called 'session'
        session = getUtility(IAuthentication)['session']
        session.logout(self.request)
        return "You are logged out."
